# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AISearchIndexResource,
    AOAIModelConfig,
    Agent,
    AgentDeletionStatus,
    AgentThread,
    AgentThreadCreationOptions,
    AgentsApiResponseFormat,
    AgentsNamedToolChoice,
    ApplicationInsightsConfiguration,
    AzureAISearchResource,
    AzureAISearchToolDefinition,
    AzureFunctionBinding,
    AzureFunctionDefinition,
    AzureFunctionStorageQueue,
    AzureFunctionToolDefinition,
    BingCustomSearchToolDefinition,
    BingGroundingToolDefinition,
    CodeInterpreterToolDefinition,
    CodeInterpreterToolResource,
    ConnectedAgentDetails,
    ConnectedAgentToolDefinition,
    CronTrigger,
    Dataset,
    Evaluation,
    EvaluationSchedule,
    EvaluationTarget,
    EvaluatorConfiguration,
    FileDeletionStatus,
    FileListResponse,
    FileSearchRankingOptions,
    FileSearchToolCallContent,
    FileSearchToolDefinition,
    FileSearchToolDefinitionDetails,
    FileSearchToolResource,
    FunctionDefinition,
    FunctionName,
    FunctionToolDefinition,
    IncompleteRunDetails,
    InputData,
    MAASModelConfig,
    MessageAttachment,
    MessageContent,
    MessageDelta,
    MessageDeltaChunk,
    MessageDeltaContent,
    MessageDeltaImageFileContent,
    MessageDeltaImageFileContentObject,
    MessageDeltaTextAnnotation,
    MessageDeltaTextContent,
    MessageDeltaTextContentObject,
    MessageDeltaTextFileCitationAnnotation,
    MessageDeltaTextFileCitationAnnotationObject,
    MessageDeltaTextFilePathAnnotation,
    MessageDeltaTextFilePathAnnotationObject,
    MessageDeltaTextUrlCitationAnnotation,
    MessageDeltaTextUrlCitationDetails,
    MessageImageFileContent,
    MessageImageFileDetails,
    MessageImageFileParam,
    MessageImageUrlParam,
    MessageIncompleteDetails,
    MessageInputContentBlock,
    MessageInputImageFileBlock,
    MessageInputImageUrlBlock,
    MessageInputTextBlock,
    MessageTextAnnotation,
    MessageTextContent,
    MessageTextDetails,
    MessageTextFileCitationAnnotation,
    MessageTextFileCitationDetails,
    MessageTextFilePathAnnotation,
    MessageTextFilePathDetails,
    MessageTextUrlCitationAnnotation,
    MessageTextUrlCitationDetails,
    MicrosoftFabricToolDefinition,
    OpenAIFile,
    OpenAIPageableListOfAgent,
    OpenAIPageableListOfAgentThread,
    OpenAIPageableListOfRunStep,
    OpenAIPageableListOfThreadMessage,
    OpenAIPageableListOfThreadRun,
    OpenAIPageableListOfVectorStore,
    OpenAIPageableListOfVectorStoreFile,
    OpenApiAnonymousAuthDetails,
    OpenApiAuthDetails,
    OpenApiConnectionAuthDetails,
    OpenApiConnectionSecurityScheme,
    OpenApiFunctionDefinition,
    OpenApiManagedAuthDetails,
    OpenApiManagedSecurityScheme,
    OpenApiToolDefinition,
    RecurrenceSchedule,
    RecurrenceTrigger,
    RequiredAction,
    RequiredFunctionToolCall,
    RequiredFunctionToolCallDetails,
    RequiredToolCall,
    ResponseFormatJsonSchema,
    ResponseFormatJsonSchemaType,
    RunCompletionUsage,
    RunError,
    RunStep,
    RunStepAzureAISearchToolCall,
    RunStepBingGroundingToolCall,
    RunStepCodeInterpreterImageOutput,
    RunStepCodeInterpreterImageReference,
    RunStepCodeInterpreterLogOutput,
    RunStepCodeInterpreterToolCall,
    RunStepCodeInterpreterToolCallDetails,
    RunStepCodeInterpreterToolCallOutput,
    RunStepCompletionUsage,
    RunStepCustomSearchToolCall,
    RunStepDelta,
    RunStepDeltaChunk,
    RunStepDeltaCodeInterpreterDetailItemObject,
    RunStepDeltaCodeInterpreterImageOutput,
    RunStepDeltaCodeInterpreterImageOutputObject,
    RunStepDeltaCodeInterpreterLogOutput,
    RunStepDeltaCodeInterpreterOutput,
    RunStepDeltaCodeInterpreterToolCall,
    RunStepDeltaDetail,
    RunStepDeltaFileSearchToolCall,
    RunStepDeltaFunction,
    RunStepDeltaFunctionToolCall,
    RunStepDeltaMessageCreation,
    RunStepDeltaMessageCreationObject,
    RunStepDeltaToolCall,
    RunStepDeltaToolCallObject,
    RunStepDetails,
    RunStepError,
    RunStepFileSearchToolCall,
    RunStepFileSearchToolCallResult,
    RunStepFileSearchToolCallResults,
    RunStepFunctionToolCall,
    RunStepFunctionToolCallDetails,
    RunStepMessageCreationDetails,
    RunStepMessageCreationReference,
    RunStepMicrosoftFabricToolCall,
    RunStepOpenAPIToolCall,
    RunStepSharepointToolCall,
    RunStepToolCall,
    RunStepToolCallDetails,
    SearchConfiguration,
    SearchConfigurationList,
    SharepointToolDefinition,
    SubmitToolOutputsAction,
    SubmitToolOutputsDetails,
    SystemData,
    TargetModelConfig,
    ThreadDeletionStatus,
    ThreadMessage,
    ThreadMessageOptions,
    ThreadRun,
    ToolConnection,
    ToolConnectionList,
    ToolDefinition,
    ToolOutput,
    ToolResources,
    Trigger,
    TruncationObject,
    UpdateCodeInterpreterToolResourceOptions,
    UpdateFileSearchToolResourceOptions,
    UpdateToolResourcesOptions,
    VectorStore,
    VectorStoreAutoChunkingStrategyRequest,
    VectorStoreAutoChunkingStrategyResponse,
    VectorStoreChunkingStrategyRequest,
    VectorStoreChunkingStrategyResponse,
    VectorStoreConfiguration,
    VectorStoreConfigurations,
    VectorStoreDataSource,
    VectorStoreDeletionStatus,
    VectorStoreExpirationPolicy,
    VectorStoreFile,
    VectorStoreFileBatch,
    VectorStoreFileCount,
    VectorStoreFileDeletionStatus,
    VectorStoreFileError,
    VectorStoreStaticChunkingStrategyOptions,
    VectorStoreStaticChunkingStrategyRequest,
    VectorStoreStaticChunkingStrategyResponse,
)

from ._enums import (  # type: ignore
    AgentStreamEvent,
    AgentsApiResponseFormatMode,
    AgentsApiToolChoiceOptionMode,
    AgentsNamedToolChoiceType,
    AuthenticationType,
    AzureAISearchQueryType,
    ConnectionType,
    DoneEvent,
    ErrorEvent,
    FilePurpose,
    FileState,
    Frequency,
    ImageDetailLevel,
    IncompleteDetailsReason,
    ListSortOrder,
    MessageBlockType,
    MessageIncompleteDetailsReason,
    MessageRole,
    MessageStatus,
    MessageStreamEvent,
    OpenApiAuthType,
    ResponseFormat,
    RunAdditionalFieldList,
    RunStatus,
    RunStepErrorCode,
    RunStepStatus,
    RunStepStreamEvent,
    RunStepType,
    RunStreamEvent,
    ThreadStreamEvent,
    TruncationStrategy,
    VectorStoreChunkingStrategyRequestType,
    VectorStoreChunkingStrategyResponseType,
    VectorStoreDataSourceAssetType,
    VectorStoreExpirationPolicyAnchor,
    VectorStoreFileBatchStatus,
    VectorStoreFileErrorCode,
    VectorStoreFileStatus,
    VectorStoreFileStatusFilter,
    VectorStoreStatus,
    WeekDays,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AISearchIndexResource",
    "AOAIModelConfig",
    "Agent",
    "AgentDeletionStatus",
    "AgentThread",
    "AgentThreadCreationOptions",
    "AgentsApiResponseFormat",
    "AgentsNamedToolChoice",
    "ApplicationInsightsConfiguration",
    "AzureAISearchResource",
    "AzureAISearchToolDefinition",
    "AzureFunctionBinding",
    "AzureFunctionDefinition",
    "AzureFunctionStorageQueue",
    "AzureFunctionToolDefinition",
    "BingCustomSearchToolDefinition",
    "BingGroundingToolDefinition",
    "CodeInterpreterToolDefinition",
    "CodeInterpreterToolResource",
    "ConnectedAgentDetails",
    "ConnectedAgentToolDefinition",
    "CronTrigger",
    "Dataset",
    "Evaluation",
    "EvaluationSchedule",
    "EvaluationTarget",
    "EvaluatorConfiguration",
    "FileDeletionStatus",
    "FileListResponse",
    "FileSearchRankingOptions",
    "FileSearchToolCallContent",
    "FileSearchToolDefinition",
    "FileSearchToolDefinitionDetails",
    "FileSearchToolResource",
    "FunctionDefinition",
    "FunctionName",
    "FunctionToolDefinition",
    "IncompleteRunDetails",
    "InputData",
    "MAASModelConfig",
    "MessageAttachment",
    "MessageContent",
    "MessageDelta",
    "MessageDeltaChunk",
    "MessageDeltaContent",
    "MessageDeltaImageFileContent",
    "MessageDeltaImageFileContentObject",
    "MessageDeltaTextAnnotation",
    "MessageDeltaTextContent",
    "MessageDeltaTextContentObject",
    "MessageDeltaTextFileCitationAnnotation",
    "MessageDeltaTextFileCitationAnnotationObject",
    "MessageDeltaTextFilePathAnnotation",
    "MessageDeltaTextFilePathAnnotationObject",
    "MessageDeltaTextUrlCitationAnnotation",
    "MessageDeltaTextUrlCitationDetails",
    "MessageImageFileContent",
    "MessageImageFileDetails",
    "MessageImageFileParam",
    "MessageImageUrlParam",
    "MessageIncompleteDetails",
    "MessageInputContentBlock",
    "MessageInputImageFileBlock",
    "MessageInputImageUrlBlock",
    "MessageInputTextBlock",
    "MessageTextAnnotation",
    "MessageTextContent",
    "MessageTextDetails",
    "MessageTextFileCitationAnnotation",
    "MessageTextFileCitationDetails",
    "MessageTextFilePathAnnotation",
    "MessageTextFilePathDetails",
    "MessageTextUrlCitationAnnotation",
    "MessageTextUrlCitationDetails",
    "MicrosoftFabricToolDefinition",
    "OpenAIFile",
    "OpenAIPageableListOfAgent",
    "OpenAIPageableListOfAgentThread",
    "OpenAIPageableListOfRunStep",
    "OpenAIPageableListOfThreadMessage",
    "OpenAIPageableListOfThreadRun",
    "OpenAIPageableListOfVectorStore",
    "OpenAIPageableListOfVectorStoreFile",
    "OpenApiAnonymousAuthDetails",
    "OpenApiAuthDetails",
    "OpenApiConnectionAuthDetails",
    "OpenApiConnectionSecurityScheme",
    "OpenApiFunctionDefinition",
    "OpenApiManagedAuthDetails",
    "OpenApiManagedSecurityScheme",
    "OpenApiToolDefinition",
    "RecurrenceSchedule",
    "RecurrenceTrigger",
    "RequiredAction",
    "RequiredFunctionToolCall",
    "RequiredFunctionToolCallDetails",
    "RequiredToolCall",
    "ResponseFormatJsonSchema",
    "ResponseFormatJsonSchemaType",
    "RunCompletionUsage",
    "RunError",
    "RunStep",
    "RunStepAzureAISearchToolCall",
    "RunStepBingGroundingToolCall",
    "RunStepCodeInterpreterImageOutput",
    "RunStepCodeInterpreterImageReference",
    "RunStepCodeInterpreterLogOutput",
    "RunStepCodeInterpreterToolCall",
    "RunStepCodeInterpreterToolCallDetails",
    "RunStepCodeInterpreterToolCallOutput",
    "RunStepCompletionUsage",
    "RunStepCustomSearchToolCall",
    "RunStepDelta",
    "RunStepDeltaChunk",
    "RunStepDeltaCodeInterpreterDetailItemObject",
    "RunStepDeltaCodeInterpreterImageOutput",
    "RunStepDeltaCodeInterpreterImageOutputObject",
    "RunStepDeltaCodeInterpreterLogOutput",
    "RunStepDeltaCodeInterpreterOutput",
    "RunStepDeltaCodeInterpreterToolCall",
    "RunStepDeltaDetail",
    "RunStepDeltaFileSearchToolCall",
    "RunStepDeltaFunction",
    "RunStepDeltaFunctionToolCall",
    "RunStepDeltaMessageCreation",
    "RunStepDeltaMessageCreationObject",
    "RunStepDeltaToolCall",
    "RunStepDeltaToolCallObject",
    "RunStepDetails",
    "RunStepError",
    "RunStepFileSearchToolCall",
    "RunStepFileSearchToolCallResult",
    "RunStepFileSearchToolCallResults",
    "RunStepFunctionToolCall",
    "RunStepFunctionToolCallDetails",
    "RunStepMessageCreationDetails",
    "RunStepMessageCreationReference",
    "RunStepMicrosoftFabricToolCall",
    "RunStepOpenAPIToolCall",
    "RunStepSharepointToolCall",
    "RunStepToolCall",
    "RunStepToolCallDetails",
    "SearchConfiguration",
    "SearchConfigurationList",
    "SharepointToolDefinition",
    "SubmitToolOutputsAction",
    "SubmitToolOutputsDetails",
    "SystemData",
    "TargetModelConfig",
    "ThreadDeletionStatus",
    "ThreadMessage",
    "ThreadMessageOptions",
    "ThreadRun",
    "ToolConnection",
    "ToolConnectionList",
    "ToolDefinition",
    "ToolOutput",
    "ToolResources",
    "Trigger",
    "TruncationObject",
    "UpdateCodeInterpreterToolResourceOptions",
    "UpdateFileSearchToolResourceOptions",
    "UpdateToolResourcesOptions",
    "VectorStore",
    "VectorStoreAutoChunkingStrategyRequest",
    "VectorStoreAutoChunkingStrategyResponse",
    "VectorStoreChunkingStrategyRequest",
    "VectorStoreChunkingStrategyResponse",
    "VectorStoreConfiguration",
    "VectorStoreConfigurations",
    "VectorStoreDataSource",
    "VectorStoreDeletionStatus",
    "VectorStoreExpirationPolicy",
    "VectorStoreFile",
    "VectorStoreFileBatch",
    "VectorStoreFileCount",
    "VectorStoreFileDeletionStatus",
    "VectorStoreFileError",
    "VectorStoreStaticChunkingStrategyOptions",
    "VectorStoreStaticChunkingStrategyRequest",
    "VectorStoreStaticChunkingStrategyResponse",
    "AgentStreamEvent",
    "AgentsApiResponseFormatMode",
    "AgentsApiToolChoiceOptionMode",
    "AgentsNamedToolChoiceType",
    "AuthenticationType",
    "AzureAISearchQueryType",
    "ConnectionType",
    "DoneEvent",
    "ErrorEvent",
    "FilePurpose",
    "FileState",
    "Frequency",
    "ImageDetailLevel",
    "IncompleteDetailsReason",
    "ListSortOrder",
    "MessageBlockType",
    "MessageIncompleteDetailsReason",
    "MessageRole",
    "MessageStatus",
    "MessageStreamEvent",
    "OpenApiAuthType",
    "ResponseFormat",
    "RunAdditionalFieldList",
    "RunStatus",
    "RunStepErrorCode",
    "RunStepStatus",
    "RunStepStreamEvent",
    "RunStepType",
    "RunStreamEvent",
    "ThreadStreamEvent",
    "TruncationStrategy",
    "VectorStoreChunkingStrategyRequestType",
    "VectorStoreChunkingStrategyResponseType",
    "VectorStoreDataSourceAssetType",
    "VectorStoreExpirationPolicyAnchor",
    "VectorStoreFileBatchStatus",
    "VectorStoreFileErrorCode",
    "VectorStoreFileStatus",
    "VectorStoreFileStatusFilter",
    "VectorStoreStatus",
    "WeekDays",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
