# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._trace_function import trace_function

__all__ = [
    "trace_function",
]
__path__ = __import__("pkgutil").extend_path(__path__, __name__)  # type: ignore
